@@ -2,6 +2,7 @@ import { css } from '@emotion/css';
 import { isEmpty } from 'lodash';
 import { useCallback, useEffect, useMemo, useState } from 'react';
 import { ConnectedProps, connect } from 'react-redux';
+import { useAsync } from 'react-use';
 
 import { DataSourceJsonData, DataSourceSettings, GrafanaTheme2 } from '@grafana/data';
 import { Trans, t } from '@grafana/i18n';
@@ -22,7 +23,7 @@ import {
   useStyles2,
 } from '@grafana/ui';
 import { ResourcePermission } from 'app/core/components/AccessControl/types';
-import { contextSrv } from 'app/core/core';
+import { contextSrv } from 'app/core/services/context_srv';
 import { DataSourceReadOnlyMessage } from 'app/features/datasources/components/DataSourceReadOnlyMessage';
 import { Team } from 'app/types/teams';
 
@@ -32,6 +33,7 @@ import { CreateTeamLBACForm, LBACFormData } from './AddTeamLBACForm';
 import { TeamRulesRow } from './TeamRulesRow';
 import { useFilterAndSort } from './hooks/useFilterAndSort';
 import { getTeamLBAC, updateTeamLBACRules } from './state/actions';
+import { selectTeamLBACDatasourceUid, selectTeamLBACRules } from './state/selectors';
 import { formatLBACRule } from './utils';
 
 export interface OwnProps {
@@ -44,7 +46,8 @@ export interface OwnProps {
 
 function mapStateToProps(state: EnterpriseStoreState) {
   return {
-    teamLBACConfig: state.teamLBAC?.teamLBACConfig,
+    teamLBACRules: selectTeamLBACRules(state),
+    storedDatasourceUid: selectTeamLBACDatasourceUid(state),
   };
 }
 
@@ -57,7 +60,8 @@ export const connector = connect(mapStateToProps, mapDispatchToProps);
 type Props = OwnProps & ConnectedProps<typeof connector>;
 
 export const TeamLBACEditorUnconnected = ({
-  teamLBACConfig,
+  teamLBACRules,
+  storedDatasourceUid,
   dataSourceConfig,
   readOnly,
   onTeamLBACUpdate,
@@ -65,12 +69,10 @@ export const TeamLBACEditorUnconnected = ({
   updateTeamLBACRules,
   items,
 }: Props) => {
-  const [teams, setTeams] = useState<Array<Pick<Team, 'name' | 'avatarUrl' | 'id' | 'uid'>>>([]);
   const [showLBACForm, setShowLBACForm] = useState(false);
   const [rolesWithoutLBAC, setRolesWithoutLBAC] = useState<Set<string>>(new Set<string>());
   const [teamsWithoutLBAC, setTeamsWithoutLBAC] = useState<Set<string>>(new Set<string>());
   const [lbacTeamsWithoutTeamPermissions, setLBACTeamsWithoutTeamPermission] = useState<Set<string>>(new Set<string>());
-  const teamLBACRules = useMemo(() => teamLBACConfig?.rules || [], [teamLBACConfig?.rules]);
 
   // Calculate the size of LBAC rules and check if it exceeds 64KB
   const lbacRulesSizeWarning = useMemo(() => {
@@ -96,6 +98,36 @@ export const TeamLBACEditorUnconnected = ({
   const styles = useStyles2(getStyles);
   const [recommendationIsOpen, setRecommendationIsOpen] = useState(false);
 
+  useEffect(() => {
+    // Fetch if datasource UID doesn't match stored UID, or if we don't have data yet
+    if (storedDatasourceUid !== dataSourceConfig.uid || teamLBACRules.length === 0) {
+      getTeamLBAC(dataSourceConfig.uid);
+    }
+  }, [dataSourceConfig.uid, getTeamLBAC, storedDatasourceUid, teamLBACRules.length]);
+
+  const teamsState = useAsync(async () => {
+    const teamUIDs = teamLBACRules.map((r) => r.teamUid).filter(Boolean);
+    const uniqueIdentifiers = [...new Set(teamUIDs)];
+
+    if (!uniqueIdentifiers?.length) {
+      return [];
+    }
+
+    const result = await getBackendSrv().get('/api/teams/search', {
+      teamUid: teamUIDs,
+    });
+    const teamsArray: Team[] = result?.teams;
+    return teamsArray.map((team) => ({
+      id: team.id,
+      uid: team.uid,
+      value: team.id,
+      name: team.name,
+      avatarUrl: team.avatarUrl,
+    }));
+  }, [teamLBACRules]);
+
+  const teams = teamsState.value ?? [];
+
   // Use the custom hook for filtering and sorting
   const {
     filterQuery,
@@ -107,35 +139,6 @@ export const TeamLBACEditorUnconnected = ({
     clearFilter,
   } = useFilterAndSort({ teamLBACRules, teams });
 
-  useEffect(() => {
-    getTeamLBAC(dataSourceConfig.uid);
-  }, [dataSourceConfig.uid, getTeamLBAC]);
-
-  useEffect(() => {
-    const fetchTeams = async () => {
-      const teamUIDs = teamLBACRules.map((r) => r.teamUid).filter(Boolean);
-      const uniqueIdentifiers = [...new Set(teamUIDs)];
-
-      if (!uniqueIdentifiers?.length) {
-        return;
-      }
-
-      const result = await getBackendSrv().get('/api/teams/search', {
-        teamUid: teamUIDs,
-      });
-      const teamsArray: Team[] = result?.teams;
-      const teams = teamsArray.map((team) => ({
-        id: team.id,
-        uid: team.uid,
-        value: team.id,
-        name: team.name,
-        avatarUrl: team.avatarUrl,
-      }));
-      setTeams(teams);
-    };
-    fetchTeams();
-  }, [teamLBACRules]);
-
   useEffect(() => {
     if (!items?.length) {
       return;
@@ -379,29 +382,31 @@ export const TeamLBACEditorUnconnected = ({
               </Stack>
             </Stack>
             {canEdit && (
-              <Stack direction={{ xs: 'column-reverse', md: 'row' }} alignItems={'flex-start'} gap={2}>
-                <LinkButton
-                  onClick={() => setShowLBACForm(!showLBACForm)}
-                  icon={showLBACForm ? 'angle-down' : 'plus'}
-                  aria-expanded={showLBACForm}
-                >
-                  {showLBACForm
-                    ? t('team-lbac.team-lbaceditor-unconnected.hide-lbac-rule', 'Hide LBAC rule')
-                    : t('team-lbac.team-lbaceditor-unconnected.add-a-lbac-rule', 'Add a LBAC rule')}
-                </LinkButton>
-                {!isEmpty(teamLBACRules) && (
-                  <FilterInput
-                    placeholder={t('team-lbac.team-lbaceditor-unconnected.filter-teams-placeholder', 'Filter teams')}
-                    value={filterQuery}
-                    onChange={setFilterQuery}
-                    width={30}
-                    escapeRegex={false}
-                    aria-label={t(
-                      'team-lbac.team-lbaceditor-unconnected.filter-teams-aria-label',
-                      'Filter LBAC rules by team name'
-                    )}
-                  />
-                )}
+              <Stack direction={{ xs: 'column' }} alignItems={'flex-start'}>
+                <Stack>
+                  <LinkButton
+                    onClick={() => setShowLBACForm(!showLBACForm)}
+                    icon={showLBACForm ? 'angle-down' : 'plus'}
+                    aria-expanded={showLBACForm}
+                  >
+                    {showLBACForm
+                      ? t('team-lbac.team-lbaceditor-unconnected.hide-lbac-rule', 'Hide LBAC rule')
+                      : t('team-lbac.team-lbaceditor-unconnected.add-a-lbac-rule', 'Add a LBAC rule')}
+                  </LinkButton>
+                  {!isEmpty(teamLBACRules) && (
+                    <FilterInput
+                      placeholder={t('team-lbac.team-lbaceditor-unconnected.filter-teams-placeholder', 'Filter teams')}
+                      value={filterQuery}
+                      onChange={setFilterQuery}
+                      width={30}
+                      escapeRegex={false}
+                      aria-label={t(
+                        'team-lbac.team-lbaceditor-unconnected.filter-teams-aria-label',
+                        'Filter LBAC rules by team name'
+                      )}
+                    />
+                  )}
+                </Stack>
                 {showLBACForm && (
                   <Box marginTop={2}>
                     <CreateTeamLBACForm onSubmit={onSubmitLBAC} datasourceType={dataSourceConfig.type} />
